/*
 * Client/server example courtesy of Marcia Perry.
 */

#include "CIF_Comm.H"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>


int main(int argc, char *argv[])
{
    const int				MAX_MSG_SZ = 512;

    int					argn;
    int					timeout = -1;
    int					cnt = 0;    
    char *				url;	
    char				mc_url[80];	
    CIF_Bool				rc;
    CIF_Comm_Connection *		cif_conn[3];	
    CIF_Byte				buf[MAX_MSG_SZ];
    CIF_Comm_Message			message;

    argn = 1;
    printf("CIF_Client: starting\n");
    while(argn < argc - 1)
    {
	if (strcmp(argv[argn], "-t") == 0)
	{
	    timeout = atoi(argv[++argn]);
	}
	else
	{
	    fprintf(stderr, "unknown argument: %s\n\n", argv[argn]);
	}
	
	argn++;
    }

    if (argc > 1)
    {
	url = new char [strlen(argv[argn])+1];
	strcpy(url, argv[argn]);
    }	
    else
    {
	printf("CIF_Client: wrong command line\n"); 
    }
    

    CIF_Comm::activate();

    /* Attempt to establish a multicast connection */
    strcpy(mc_url, "x-cif-uum://224.35.36.37:5556/ttl=16+loopback=false");
    printf("CIF_Client establishing multicast connection for URL=%s\n",mc_url);
    if (timeout >= 0)
    {
	cif_conn[1] = CIF_Comm_Factory::get_Connection(mc_url, timeout);
    }
    else
    {
	cif_conn[1] = CIF_Comm_Factory::get_Connection(mc_url);
    }
    
    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    assert(cif_conn[1] != CIF_NULL);
    printf("CIF_Client: multicast connection established.\n\n");
    
    for (;;)
    {
	printf("CIF_Client: url: %s, cnt = %d\n", url, cnt);

	/* Attempt to establish a unicast connection */
	printf("CIF_Client establishing unicast connection for URL=%s\n",url);
	if (timeout >= 0)
	{
	    cif_conn[0] = CIF_Comm_Factory::get_Connection(url, timeout);
	}
	else
	{
	    cif_conn[0] = CIF_Comm_Factory::get_Connection(url);
	}
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	assert(cif_conn[0] != CIF_NULL);
	printf("CIF_Client: unicast connection established.\n");
	
        printf("CIF_Client: sending message %d\n", cnt);
	sprintf((char *)buf, "data for message %d", cnt);
	rc = cif_conn[0]->send(buf,strlen((char *)buf) + 1,cnt);
	assert(rc == true);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	cnt++;
	   
	/* Receive multicast messages.  */
	while (cif_conn[1]->poll() == false)
	{
	    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	}

	rc = cif_conn[1]->receive(message);
	printf("CIF_Client: rec'd uum msg: \"%s\n",message.get_Data());

	/* Close unicast connection and delete object  */
	printf("Communication complete.  Closing connection.\n");
	rc = cif_conn[0]->close();
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	assert(rc == true);
	delete cif_conn[0];  
    } 

    printf("CIF_Client: closing multicast connection and exiting\n");	
    rc = cif_conn[1]->close();
    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    assert(rc == true);
    delete cif_conn[1];
    CIF_Comm::deactivate();
    exit(0);
}
