/******************************************************************************
CIF_I_Comm_Listener.H

Description:

  CIF_Comm Listener Class Implementation

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Comm-C++-Current/library/CIF_I_Comm_Listener.H,v $
  $Date: 1998/10/09 01:00:34 $
  $Revision: 1.5 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

#if !defined(CIF_INCLUDE_I_COMM_LISTENER)
#define CIF_INCLUDE_I_COMM_LISTENER 1

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_Comm.H"
#include "CIF_I_Comm_Endpoint.H"
#include "globus_nexus.h"
#include "globus_common.h"

 
/******************************************************************************
		       Define CIF_I_Comm_Listener class
******************************************************************************/
class
CIF_I_Comm_Listener
: public CIF_Comm_Listener,
  public CIF_Toolkit_Event_Listener
{
    /**************************************************************************
				    API methods
    **************************************************************************/

    protected:
    CIF_I_Comm_Listener();

    protected: virtual
    CIF_Bool
    open(
	ARG_IN const unsigned short		port,
	ARG_IN const Properties &		properties);

    protected: virtual
    CIF_Comm_Connection *
    get_Connection();
    
    protected: virtual
    CIF_Comm_Connection *
    get_Connection(
	ARG_IN const
	CIF_Toolkit_Timeout &			timeout);
    
    protected: virtual
    CIF_Comm_Connection *
    get_Connection(
	ARG_IN const
	CIF_Comm_Connection::Properties &	properties);
    
    protected: virtual
    CIF_Comm_Connection *
    get_Connection(
	ARG_IN const
	CIF_Comm_Connection::Properties &	properties,
	ARG_IN const
	CIF_Toolkit_Timeout &			timeout);
    
    public: virtual
    CIF_Bool
    enable_Event_Listener();

    public: virtual
    CIF_Bool
    disable_Event_Listener();

    protected: virtual
    const char *
    get_URL();

    protected: virtual
    CIF_Bool
    close();

    protected: virtual
    ~CIF_I_Comm_Listener();
    
    /**************************************************************************
			        Derived API methods
    **************************************************************************/
    protected: virtual
    CIF_Comm_Connection *
    open_Connection(
	nexus_buffer_t &			buffer,
	ARG_IN const
	CIF_Comm_Connection::Properties &	properties,
	ARG_IN const CIF_Toolkit_Timeout &	timeout) = 0;

    protected: virtual
    char *
    get_URL_Scheme() = 0;

    
    /**************************************************************************
		      Internal methods and data structures
    **************************************************************************/
    private:
    enum Object_State
    {
	OBJECT_STATE_INACTIVE,
	OBJECT_STATE_ACTIVE,
	OBJECT_STATE_DELETED
    };

    public:
    void
    handle_Event(
	ARG_INMOD CIF_Toolkit_Event &		ep_event);

    private:
    CIF_Comm_Connection *
    l_get_Connection(
	ARG_IN const
	CIF_Comm_Connection::Properties &	properties,
	ARG_IN const
	CIF_Toolkit_Timeout &			timeout);

    private:
    CIF_Bool
    l_close(
	CIF_Toolkit_Monitor &			monitor);
    
    private:
    CIF_Comm_Status
    l_verify_Object_State(
	ARG_IN const Object_State &		desired_state);
    

    /**************************************************************************
			       Internal data members
    **************************************************************************/
    private:

    /* mutex to protect listener data structures */
    CIF_Toolkit_Mutex				l_mutex;

    /* endpoint for receiving connection establishment information */
    CIF_I_Comm_Endpoint				l_ep;

    /* event listener to notify when a connection request arrives */
    CIF_Toolkit_Event_Listener *		l_event_listener;

    /* synchronization variables to ensure that a close doesn't occur while
       other threads using the object are waiting for blocking operations to
       complete. */
    CIF_Toolkit_Cond				l_close_cond;
    volatile int				l_blocking_cnt;
    volatile CIF_Bool				l_close_pending;
    
    /* URL used to contact this listener */
    char *					l_url;
    
    /* state of the connection */
    volatile Object_State			l_object_state;
};

#endif /* CIF_INCLUDE_I_COMM_LISTENER */
